import type { TextStyle } from 'react-native'
import transform from 'css-to-react-native'

export interface HighlighterStyleSheet {
  [key: string]: TextStyle
}

export type ReactStyle = Record<string, TextStyle>

// Helper function to convert hex color to RGB
function hexToRgb(hex: string): { r: number, g: number, b: number } | null {
  const result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex)
  return result
    ? {
        r: Number.parseInt(result[1]!, 16),
        g: Number.parseInt(result[2]!, 16),
        b: Number.parseInt(result[3]!, 16),
      }
    : null
}

// Helper function to normalize color values
function normalizeColor(color: string | undefined): string {
  if (!color)
    return 'transparent'

  // Type guard to ensure color is string before we call string methods
  if (typeof color === 'string' && color.startsWith('#')) {
    const rgb = hexToRgb(color)
    if (rgb) {
      return `rgb(${rgb.r}, ${rgb.g}, ${rgb.b})`
    }
  }
  return color
}

export function convertTokenColorsToReactStyle(tokenColors: any[]): ReactStyle {
  const styleMap: ReactStyle = {}

  // Add default styles
  styleMap.default = {
    color: '#e1e4e8',
    backgroundColor: 'transparent',
  }

  tokenColors.forEach((token) => {
    if (!token.settings || !token.scope)
      return

    const scopes = Array.isArray(token.scope) ? token.scope : [token.scope]
    const styleEntries: Array<[string, string]> = []

    // Only add styles if they exist
    if (token.settings.foreground) {
      styleEntries.push(['color', normalizeColor(token.settings.foreground)])
    }
    if (token.settings.background) {
      styleEntries.push(['backgroundColor', normalizeColor(token.settings.background)])
    }
    if (token.settings.fontStyle) {
      if (token.settings.fontStyle.includes('italic')) {
        styleEntries.push(['fontStyle', 'italic'])
      }
      if (token.settings.fontStyle.includes('bold')) {
        styleEntries.push(['fontWeight', 'bold'])
      }
    }

    if (styleEntries.length > 0) {
      const transformedStyle = transform(styleEntries)

      scopes.forEach((scope: string) => {
        if (typeof scope === 'string') {
          styleMap[scope] = { ...(styleMap[scope] || {}), ...transformedStyle }
        }
      })
    }
  })

  return styleMap
}

export function getRNStylesFromShikiStyle(theme: any): HighlighterStyleSheet {
  if (!theme || !theme.tokenColors) {
    console.error('Provided theme does not contain \'tokenColors\'.')
    return {}
  }

  try {
    // Convert theme colors to React Native styles
    const styles = convertTokenColorsToReactStyle(theme.tokenColors)

    // Add theme-specific base styles if they exist
    if (theme.colors) {
      styles.base = {
        color: normalizeColor(theme.colors['editor.foreground']),
        backgroundColor: normalizeColor(theme.colors['editor.background']),
      }
    }

    return styles
  }
  catch {
    console.error('Error converting theme styles')
    return {}
  }
}
